import argparse

from subwiz.main import (
    download_files,
    run_inference,
    run_resolution,
)
from subwiz.types import (
    device_type,
    input_domains_file_type,
    output_file_type,
    positive_int_type,
    temperature_type,
)


parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
# parser.print_usage = parser.print_help
parser.add_argument(
    "-i",
    "--input-file",
    help="file containing new-line-separated subdomains.",
    dest="input_file",
    required=True,
    type=input_domains_file_type,
)
parser.add_argument(
    "-o",
    "--output-file",
    help="output file to write new-line separated subdomains to.",
    dest="output_file",
    type=output_file_type,
)
parser.add_argument(
    "-n",
    "--num_predictions",
    help="number of subdomains to predict.",
    dest="num_predictions",
    default=500,
    type=positive_int_type,
)
parser.add_argument(
    "--no-resolve",
    help="do not resolve the output subdomains. ",
    dest="no_resolve",
    action="store_true",
)
parser.add_argument(
    "--force-download",
    help="download model and tokenizer files, even if cached. ",
    dest="force_download",
    action="store_true",
)
parser.add_argument(
    "-t",
    "--temperature",
    help="add randomness to the model, recommended ≤ 0.3)",
    dest="temperature",
    default=0.0,
    type=temperature_type,
)
parser.add_argument(
    "-d",
    "--device",
    help="hardware to run the transformer model on.",
    dest="device",
    default="auto",
    choices=["auto", "cpu", "cuda", "mps"],
    type=device_type,
)
parser.add_argument(
    "-q",
    "--max_new_tokens",
    help="maximum length of predicted subdomains in tokens.",
    dest="max_new_tokens",
    default=10,
    type=positive_int_type,
)
parser.add_argument(
    "--resolution_concurrency",
    help="number of concurrent resolutions.",
    dest="resolution_lim",
    default=128,
    type=positive_int_type,
)
args = parser.parse_args()


class bcolors:
    HEADER = "\033[95m"
    OKBLUE = "\033[94m"
    OKCYAN = "\033[96m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"


hello_message = """
███████╗██╗   ██╗██████╗     ██╗    ██╗██╗███████╗
██╔════╝██║   ██║██╔══██╗    ██║    ██║██║╚══███╔╝
███████╗██║   ██║██████╔╝    ██║ █╗ ██║██║  ███╔╝ 
╚════██║██║   ██║██╔══██╗    ██║███╗██║██║ ███╔╝  
███████║╚██████╔╝██████╔╝    ╚███╔███╔╝██║███████╗
╚══════╝ ╚═════╝ ╚═════╝      ╚══╝╚══╝ ╚═╝╚══════╝"""


def print_hello():
    print(f"{bcolors.OKGREEN}{hello_message}{bcolors.ENDC}", flush=True)


def print_log(msg: str, end="\n"):
    print(f"{bcolors.OKCYAN}{msg}{bcolors.ENDC}", flush=True, end=end)


def print_progress_dot():
    print_log(".", end="")


def main():
    print_hello()

    model_path, tokenizer_path = download_files(force_download=args.force_download)

    print_log("running inference", end="")
    predictions = run_inference(
        input_domains=args.input_file,
        device=args.device,
        model_path=model_path,
        tokenizer_path=tokenizer_path,
        num_predictions=args.num_predictions,
        max_new_tokens=args.max_new_tokens,
        temperature=args.temperature,
        on_inference_iteration=print_progress_dot,
    )
    print_log("")  # end line

    if not args.no_resolve:
        print_log("resolving subdomains...")
        predictions = run_resolution(predictions, resolution_lim=args.resolution_lim)

    output = "\n".join(sorted(predictions))

    if args.output_file:
        with open(args.output_file, "w") as f:
            f.write(output)
    else:
        print(output)


if __name__ == "__main__":
    main()



from argparse import ArgumentTypeError
import os

from subwiz.types import input_domains_file_type


def test_file_doesnt_exist():
    try:
        input_domains_file_type("nonexistent_file.txt")
        assert False
    except ArgumentTypeError:
        pass


def test_good_file():
    test_file_name = ".test_input_domains_file.txt"
    with open(test_file_name, "w") as f:
        f.write("admin.hadrian.io\ntest.hadrian.io")
    input_domains_file_type(test_file_name)
    os.remove(test_file_name)


def test_empty_file():
    test_file_name = ".test_input_domains_file.txt"
    with open(test_file_name, "w") as f:
        f.write("")
    try:
        input_domains_file_type(test_file_name)
        os.remove(test_file_name)
        assert False
    except ArgumentTypeError:
        os.remove(test_file_name)
        pass


def test_non_domain_file():
    test_file_name = ".test_input_domains_file.txt"
    with open(test_file_name, "w") as f:
        f.write("testing\ntesting")
    try:
        input_domains_file_type(test_file_name)
        os.remove(test_file_name)
        assert False
    except ArgumentTypeError:
        os.remove(test_file_name)
        pass



import os
import subprocess


def test_():
    with open("README.md", "r") as f:
        readme = f.read()

    result = subprocess.run(
        ["python", "-m", "subwiz.cli", "-h"], capture_output=True, text=True
    )
    help_text = result.stdout

    assert help_text in readme



import argparse
import os

import tldextract
import torch


class Domain:
    def __init__(self, value: str):
        extract = tldextract.extract(value)
        self.subdomain, self.domain, self.suffix = (
            extract.subdomain,
            extract.domain,
            extract.suffix,
        )

        if not bool(self.domain and self.suffix):
            raise ValueError(f"not a valid domain: {value}")

    @property
    def apex_domain(self) -> str:
        return self.domain + "." + self.suffix

    def __str__(self):
        if not self.subdomain:
            return self.apex_domain
        return self.subdomain + "." + self.apex_domain


def positive_int_type(value: str | int) -> int:
    try:
        ivalue = int(value)
    except ValueError:
        raise argparse.ArgumentTypeError(f"use a positive integer: {value}")
    if ivalue <= 0:
        raise argparse.ArgumentTypeError(f"use a positive integer: {value}")
    return ivalue


def input_domains_file_type(value: str | os.PathLike) -> list[Domain]:
    if not isinstance(value, (os.PathLike, str)):
        raise argparse.ArgumentTypeError(
            f"use a pathlike input for file path:  {value}"
        )
    if not os.path.exists(value):
        raise argparse.ArgumentTypeError(f"file not found: {value}")

    with open(value) as f:
        input_domains = f.read().split()

    return input_domains_type(input_domains)


def input_domains_type(value: list[str]) -> list[Domain]:
    if len(value) == 0:
        raise argparse.ArgumentTypeError(f"empty input domains")

    value = set(value)

    domains, invalid_domains = [], []
    for dom in value:
        try:
            domains.append(Domain(dom))
        except ValueError:
            invalid_domains.append(dom)

    if invalid_domains:
        raise argparse.ArgumentTypeError(
            f"invalid input domains: {sorted(invalid_domains)}"
        )

    apex_domains = sorted({dom.apex_domain for dom in domains})
    if len(apex_domains) != 1:
        raise argparse.ArgumentTypeError(
            f"all input domains must have same apex. found: {apex_domains}"
        )

    if not any(dom.subdomain for dom in domains):
        raise argparse.ArgumentTypeError(
            f"input should include at least one subdomain: {list(value)}"
        )

    return domains


def output_file_type(value: str | os.PathLike | None) -> os.PathLike | None:
    if value is None:
        return value

    if not isinstance(value, (os.PathLike, str)):
        raise argparse.ArgumentTypeError(
            f"use a pathlike input for file path:  {value}"
        )
    return value


def temperature_type(value: str | float) -> float:
    try:
        fvalue = float(value)
    except ValueError:
        raise argparse.ArgumentTypeError(f"not a valid float: {value}")
    if not 0 <= fvalue <= 1:
        raise argparse.ArgumentTypeError(f"use 0 ≤ temperature ≤ 1: {value}")
    return fvalue


def concurrency_type(value: str | int) -> int:
    try:
        ivalue = int(value)
    except ValueError:
        raise argparse.ArgumentTypeError(f"not a valid int: {value}")
    if not 1 <= ivalue <= 256:
        raise argparse.ArgumentTypeError(f"use 1 ≤ concurrency ≤ 256: {value}")
    return value


def device_type(value: str) -> str:
    if not isinstance(value, str):
        raise argparse.ArgumentTypeError(f"use a string for the device")

    if value not in ["auto", "cpu", "cuda", "mps"]:
        raise argparse.ArgumentTypeError(
            f'device should be in ["auto", "cpu", "cuda", "mps"]: {value}'
        )

    if value != "auto":
        return value

    elif torch.cuda.is_available():
        return "cuda"
    elif torch.backends.mps.is_available():
        return "mps"
    return "cpu"



import asyncio

import aiodns
import idna.core


NAME_SERVERS = ["1.1.1.1", "1.0.0.1", "8.8.8.8"]
TIMEOUT = 3
TRIES = 1
DNS_RECORD = "A"


async def is_registered(
    permutation: str, resolver: aiodns.DNSResolver, semaphore: asyncio.Semaphore
) -> bool:
    async with semaphore:
        try:
            await resolver.query(permutation, DNS_RECORD)
            return True
        except idna.IDNAError:
            return False
        except aiodns.error.DNSError:
            return False


async def is_registered_bulk(domains_to_check: list[str], limit: int) -> list[str]:
    semaphore = asyncio.Semaphore(limit)
    resolver = aiodns.DNSResolver(
        nameservers=NAME_SERVERS, timeout=TIMEOUT, tries=TRIES
    )

    tasks = [is_registered(dom, resolver, semaphore) for dom in domains_to_check]
    results = await asyncio.gather(*tasks)
    registered_domains = [
        dom for dom, is_reg in zip(domains_to_check, results) if is_reg
    ]

    return registered_domains



from subwiz.main import run_resolution


def test_():
    input_domains = ["api.hadrian.io", "random_test.hadrian.io", "app.hadrian.io"]
    registered_domains = run_resolution(input_domains, resolution_lim=10)
    assert registered_domains == ["api.hadrian.io", "app.hadrian.io"]



import argparse
import asyncio
import os
import re
from typing import Callable

from huggingface_hub import hf_hub_download
import tldextract
import torch
from transformers import PreTrainedTokenizerFast

from subwiz.model import GPT
from subwiz.resolve import is_registered_bulk
from subwiz.types import (
    Domain,
    input_domains_type,
    device_type,
    positive_int_type,
    temperature_type,
    concurrency_type,
)


MODEL_REPO = "HadrianSecurity/subwiz"
MODEL_FILE = "model.pt"
TOKENIZER_FILE = "tokenizer.json"
CONFIG_FILE = "config.json"


def download_files(force_download: bool) -> tuple[str, str]:
    model_path = hf_hub_download(
        repo_id=MODEL_REPO, filename=MODEL_FILE, force_download=force_download
    )
    tokenizer_path = hf_hub_download(
        repo_id=MODEL_REPO, filename=TOKENIZER_FILE, force_download=force_download
    )
    hf_hub_download(
        repo_id=MODEL_REPO, filename=CONFIG_FILE, force_download=force_download
    )
    return model_path, tokenizer_path


def run_inference(
    input_domains: list[Domain],
    device: str,
    model_path: str,
    tokenizer_path: str,
    num_predictions: int,
    max_new_tokens: int,
    temperature: float,
    on_inference_iteration: Callable = None,
) -> list[str]:

    gpt_model = GPT.from_checkpoint(
        model_path, device=device, tokenizer_path=tokenizer_path
    )
    tokenizer = PreTrainedTokenizerFast(
        tokenizer_file=tokenizer_path, clean_up_tokenization_spaces=True
    )

    subs = [dom.subdomain for dom in input_domains]
    tokenizer_input = ",".join(sorted(subs)) + "[DELIM]"

    x = tokenizer.encode(tokenizer_input)
    x = [1] * (gpt_model.config.block_size - len(x)) + x
    x = torch.tensor(x)

    predictions = gpt_model.generate(
        x,
        max_new_tokens=max_new_tokens,
        topn=num_predictions,
        on_iteration=on_inference_iteration,
        temperature=temperature,
    )
    predictions = predictions.int().tolist()

    predictions = {
        tokenizer.decode(pred).replace(" ", "").rsplit("[DELIM]", 1)[1]
        for pred in predictions
    }
    predictions = {sub + "." + input_domains[0].apex_domain for sub in predictions}
    predictions = {re.sub(r"\.+", ".", dom) for dom in predictions}
    predictions = predictions - {str(dom) for dom in input_domains}

    return sorted(predictions)


def run_resolution(predictions: list[str], resolution_lim: int):
    registered_domains = asyncio.run(is_registered_bulk(predictions, resolution_lim))
    return registered_domains


def run(
    input_domains: list[str],
    device: str = "auto",
    num_predictions: int = 500,
    max_new_tokens: int = 10,
    temperature: float = 0,
    resolution_concurrency: int = 128,
    no_resolve: bool = False,
    force_download: bool = False,
) -> list[str]:

    input_domains = input_domains_type(input_domains)
    device = device_type(device)
    num_predictions = positive_int_type(num_predictions)
    max_new_tokens = positive_int_type(max_new_tokens)
    temperature = temperature_type(temperature)
    resolution_concurrency = concurrency_type(resolution_concurrency)

    model_path, tokenizer_path = download_files(force_download)

    predictions = run_inference(
        input_domains=input_domains,
        device=device,
        model_path=model_path,
        tokenizer_path=tokenizer_path,
        num_predictions=num_predictions,
        max_new_tokens=max_new_tokens,
        temperature=temperature,
    )

    if no_resolve:
        return predictions

    resolving_predictions = run_resolution(
        predictions, resolution_lim=resolution_concurrency
    )
    return resolving_predictions



from argparse import ArgumentTypeError

from subwiz.main import device_type


def test_():
    assert device_type("auto") in ["mps", "cuda", "cpu"]
    assert device_type("mps") == "mps"
    assert device_type("cuda") == "cuda"
    assert device_type("cpu") == "cpu"

    try:
        device_type("test")
        assert False
    except ArgumentTypeError:
        pass



from setuptools import setup, find_packages

setup(
    name='subwiz',
    version='0.1',
    packages=find_packages(),
    install_requires=[
        "aiodns>=1.0.0",
        "huggingface-hub>=0.5.0",
        "pydantic>=1.10.0",
        "tldextract>=3.0.0",
        "torch>=2.3.0",
        "transformers>=4.34.0"
    ],
    entry_points={
        'console_scripts': [
            'subwiz = subwiz:cli.main'
        ]
    }
)
